/*
 * Arc.java
 *
 * Created on 12. 3. 2003
 */

/**
 *
 * @author  Martin Cadik - cadikm@cs.felk.cvut.cz
 * @version 1.0
 */

import java.util.*;
import java.awt.*;
import java.awt.geom.*;

public class Arc extends Wall {
    public static final int OUTER_WALL_ARC=7;
    /** Initial "radius" of arc */
    public static final int INITIAL_ARC_RADIUS=100;

    /** width of an ellipse ("radius") */
    public int radius=INITIAL_ARC_RADIUS;

    public Arc(int x, int y, int r, int width, int height){
        super(x, y, width, height);
        this.radius=r;
    }

    
    /** 
        Creates new Arc
        thickness is in milimeters, left upper point is lexicographically sorted
        against right lower point 
    */
    public Arc(int x1, int y1, int x2, int y2, int r, int type, int thickness){
        this(x1, y1, r, (int)(new Point2D.Float(x1, y1).distance(x2,y2)), Transform.mm2point(thickness));
        this.thickness=thickness;
        if(!( (x1<x2) || ((x1==x2)&&(y1<=y2)) )) {
            this.x=x2; this.y=y2;
            x2=x1; y2=y1; x1=this.x; y1=this.y; 
        
        }                
        this.type=type;
        rot=Math.atan((double)(y2-y1)/(double)(x2-x1));
        if(x2<x1) rot+=Math.PI;
        this.x2=x2;
        this.y2=y2;
    }

    /**
        Defines how the arc looks
    */
    private Area arcShape(){
        Ellipse2D eOuter=new Ellipse2D.Double(x, y-Math.abs(radius/2), width, Math.abs(radius));
        Ellipse2D eInner=new Ellipse2D.Double(x+height, y-Math.abs(radius/2)+height, width-2*height, Math.abs(radius)-2*height);
        Rectangle2D rClip=null;
        if(radius<0) rClip=new Rectangle2D.Double(x, y-Math.abs(radius/2), width, Math.abs(radius/2));
        else rClip=new Rectangle2D.Double(x, y, width, Math.abs(radius/2)+2);
        Area arc=new Area(eOuter);
        arc.subtract(new Area(eInner));
        arc.subtract(new Area(rClip));     
        return (arc);
    }
    
    public boolean contains(double xx, double yy){
        AffineTransform at=new AffineTransform();
        at.setToTranslation(x,y);
        at.rotate(rot);
        at.translate(-x,-y);             
        Shape s=at.createTransformedShape(arcShape());
        return(s.contains(xx, yy));
    }
    
    public Object clone(){
        Arc a=new Arc(x, y, x2, y2, radius, type, thickness);
        return(a);
    }
    
    public void draw(Graphics2D g2) {
        AffineTransform at=new AffineTransform();
        at.setToTranslation(x,y);
        at.rotate(rot);
        at.translate(-x,-y);
        g2.setPaint(Color.gray);
        //g2.draw(this);
        Area arc=arcShape();
        g2.draw(at.createTransformedShape(arc));
        g2.setPaint(Color.lightGray);       
        at.translate(x,y);       
/*        for(int i=0; i<holes.size(); i++) {        
            Rectangle r=(Rectangle)holes.get(i);
            at.translate(x+r.x,y);
            g2.draw(at.createTransformedShape(r));
            at.translate(-x-r.x,-y);          
        }              
*/
    }//draw

    
    public void fill(Graphics2D g2) {
        AffineTransform at=new AffineTransform();
        at.setToTranslation(x,y);
        at.rotate(rot);
        at.translate(-x,-y);
        g2.setPaint(Color.red);
        Area arc=arcShape();
        g2.fill(at.createTransformedShape(arc));
        g2.setPaint(Color.blue);
        at.translate(x,y);       
/*        for(int i=0; i<holes.size(); i++) {
            Rectangle r=(Rectangle)holes.get(i);
            at.translate(x+r.x,y);
            g2.fill(at.createTransformedShape(r));
            at.translate(-x-r.x,-y);
        }
*/
    }//fill


    public void drawFill(Graphics2D g2) {
        AffineTransform at=new AffineTransform();
        at.setToTranslation(x,y);
        at.rotate(rot);
        at.translate(-x,-y);        
        Area arc=arcShape();        
        g2.setPaint(Color.gray);
        g2.draw(at.createTransformedShape(arc));
        g2.setPaint(Color.red);
        g2.fill(at.createTransformedShape(arc));               
        at.translate(x,y);
    }//drawFill

    
    public void drawFillSelected(Graphics2D g2) {
        AffineTransform at=new AffineTransform();
        at.setToTranslation(x,y);
        at.rotate(rot);
        at.translate(-x,-y);        
        Area arc=arcShape();
        g2.setPaint(Color.gray);
        g2.draw(at.createTransformedShape(arc));
        g2.setPaint(Color.green);
        g2.fill(at.createTransformedShape(arc));
        at.translate(x,y);       
    }//drawFillSelected

}//Arc