/*
 * EditPanel.java
 *
 * Created on 12. 3. 2003
 */

/**
 *
 * @author  Martin Cadik - cadikm@cs.felk.cvut.cz
 * @version 1.0
 */
import java.util.Vector;
import java.util.Iterator;
import java.util.ListIterator;
import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;

public class EditPanel extends JPanel implements MouseListener, MouseMotionListener{
    /** Minimal wall length */
    public static final int MIN_WALL_LENGHT=3;
    /** MapEditApplet */	
    private MapEdit mapEditApplet=null;
    /** Level of floor which this EditPanel contains */	
    public int floorLevel=0;
    /** Initial type of the wall */
    public int wallType=Wall.WALL_ROAD;
    /** Initial width of the wall */
    public int wallWidth=270;
    /** Mode - editing / drawing */
    public boolean edit=false;
    /** Actually selected Shape */    
    Shape selectedShape=null;    
    /** Just editing arc */
    Arc editArc=null;
    
    /** Array of Shapes */
    public Vector shapes=new Vector();    
    /** Holds the coordinates of the user's last mousePressed event. */
    int lastX, lastY;
    int tempX, tempY;
    
    
    //!!
    boolean firstTime = true;
    Rectangle area;                


    /** Constructor */
    public EditPanel(MapEdit app, int fl) {
        mapEditApplet=app;
        floorLevel=fl;
        setBackground(Color.white);
        addMouseMotionListener(this);
        addMouseListener(this);       
        //setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
    }//EditPanel

    
    /** Removes all the shapes */
    public void clean(){
        shapes.clear();
        repaint();
    }//clean

    
    /** Removes the selected Shape */
    public void delete(){
        if(selectedShape==null) return;
        Iterator iter=shapes.iterator();                
        while(iter.hasNext()){         
            Shape s=(Shape)iter.next();
            if(s==selectedShape){
                shapes.remove(s); //!! je treba vyhazet i vyplne otvoru !!
                break;
            }
        }//while                            
    }//delete

    
    /**
        Checks whether or not the cursor is inside of some shape 
        If it it is then selectedShape is set else it is set to null
    */
    private boolean selectShape(MouseEvent e){
        ListIterator iter=shapes.listIterator(shapes.size());
        Shape s=null;
        while(iter.hasPrevious()){
            s=(Shape)iter.previous();                       
            if ( s.contains(e.getX(), e.getY()) ) {                                    
                selectedShape=s;
                /*System.out.println("Selected shape: "+selectedShape);
                if(selectedShape instanceof Wall) System.out.println("Wall type: "+((Wall)selectedShape).type+" wall width: "+((Wall)selectedShape).thickness);                
                if(selectedShape instanceof Wall) System.out.println("rot: "+((Wall)selectedShape).rot);
                */
                
                lastX = (int)s.getBounds2D().getX() - e.getX();
                lastY = (int)s.getBounds2D().getY() - e.getY();                                       
                return(true);                    
            }
        }//while
        selectedShape=null;
        return(false);
    }

    /**
        Returns Vector of actual outer walls
    */
    public Vector outerWalls(){
        Vector v=new Vector();
        Shape s=null;
        Iterator iter=shapes.iterator();        
        while(iter.hasNext()){         
            s=(Shape)iter.next();
            if(s instanceof Wall){ 
                if( (((Wall)s).type==Wall.WALL_ROAD) || (((Wall)s).type==Arc.OUTER_WALL_ARC) )
                    v.add(s);
            }//wall
        }//while              
        return(v);
    }//outerWalls

    
    /** Handles the event of the user pressing down the mouse button */
    public void mousePressed(MouseEvent e){        
        if( (e.getModifiers() & InputEvent.BUTTON3_MASK)== InputEvent.BUTTON3_MASK )
            return;
        
        if(edit){//edit mode
            selectShape(e);                                  
            updateParameters(selectedShape, e.getX(), e.getY());
            updateLocation(e);                                                    
        }//edit mode
        
        else{//if no shape is selected - begin drawing new one
            selectedShape=null;            
            if( mapEditApplet.checkBoxGrid.isSelected() ) {
                lastX = gridCoord(e.getX());
                lastY = gridCoord(e.getY());
            }
            else {
                lastX = e.getX();
                lastY = e.getY();
            }
            tempX=lastX;
            tempY=lastY;
        }//drawing mode
    }

    
    // Handles the event of a user dragging the mouse while holding
   // down the mouse button.
    public void mouseDragged(MouseEvent e){
        if( (e.getModifiers() & InputEvent.BUTTON3_MASK)== InputEvent.BUTTON3_MASK )
            return;
        
        if(edit){//edit mode            
            if ( selectedShape!=null ) {
                updateParametersDrag(selectedShape);
                updateLocation(e);
                //!!! urychleni: misto prekreslovani cele sceny pouzit take XORmode
                updateCoordinates((int)selectedShape.getBounds2D().getX(), (int)selectedShape.getBounds2D().getY());
            }
        }//edit mode
        else {//drawing mode
            if( wallType!=Text.TEXT ){
                //temporary drawing
                int x,y;
                if(mapEditApplet.checkBoxGrid.isSelected()) {
                    x=gridCoord(e.getX());
                    y=gridCoord(e.getY());
                }
                else {
                    x=e.getX();
                    y=e.getY();
                }
                updateCoordinates(x, y);
                Graphics2D g2 = (Graphics2D)getGraphics();

    /*          Line2D line; 
                //redraw the old line
                line = new Line2D.Float(lastX, lastY, tempX, tempY);            
                g2.setXORMode(Color.yellow);
                g2.draw(line);
                //draw new line
                line = new Line2D.Float(lastX, lastY, x, y);            
                g2.setXORMode(Color.yellow);
                //g2.setPaint(Color.yellow);
                g2.draw(line);
                tempX=x; 
                tempY=y;
    */
                //redraw the old line                
                switch(wallType){
                    case Wall.WALL_ROAD:
                    case Wall.WALL_RIVER:
                    case Wall.WALL_BRIDGE:     
                        Wall wall = new Wall(mapEditApplet.ID, lastX, lastY, tempX, tempY, wallType, wallWidth);
                        g2.setXORMode(Color.yellow);
                        wall.draw(g2);
                        //draw new line
                        wall = new Wall(mapEditApplet.ID, lastX, lastY, x, y, wallType, wallWidth);
                        g2.setXORMode(Color.yellow);
                        wall.draw(g2);
                        tempX=x; 
                        tempY=y;
                        updateParametersDrag(wall);                
                        break;
                    
                    case Arc.OUTER_WALL_ARC:                        
                        Arc arc = new Arc(lastX, lastY, tempX, tempY, Arc.INITIAL_ARC_RADIUS, wallType, wallWidth);
                        g2.setXORMode(Color.yellow);
                        arc.draw(g2);
                        //draw new line
                        arc = new Arc(lastX, lastY, x, y, Arc.INITIAL_ARC_RADIUS, wallType, wallWidth);                        
                        g2.setXORMode(Color.yellow);
                        arc.draw(g2);
                        tempX=x; 
                        tempY=y;
                        updateParametersDrag(arc);
                        break;                      
                }//switch wallType
            }//!Wall !Door
        }//drawing mode
    }

    
    // Handles the event of a user releasing the mouse button.
    public void mouseReleased(MouseEvent e){
        if( (e.getModifiers() & InputEvent.BUTTON3_MASK)== InputEvent.BUTTON3_MASK )
            return;

        if(edit){//edit mode            
            // Checks whether or not the cursor is inside of the rectangle
            // when the user releases the mouse button.
            if (selectedShape!=null){
                if(selectedShape.contains(e.getX(), e.getY()) ) updateLocation(e);
            }
        }//edit mode
        else{//drawing mode
            if( wallType!=Text.TEXT ){
                int x,y;
                if(mapEditApplet.checkBoxGrid.isSelected()) {
                    x=gridCoord(e.getX());
                    y=gridCoord(e.getY());
                }
                else {
                    x=e.getX();
                    y=e.getY();
                }         
                Point2D p=new Point2D.Float(lastX, lastY);
                                   
                if(p.distance(x,y)>=MIN_WALL_LENGHT)
                switch(wallType){
                    case Wall.WALL_ROAD:
                    case Wall.WALL_RIVER:
                    case Wall.WALL_BRIDGE:
                        Wall w=null;                
                        shapes.add(w=new Wall(++mapEditApplet.ID, lastX, lastY, x, y, wallType, wallWidth));
                        mapEditApplet.updateWallOnDilewa(mapEditApplet.ID, lastX, lastY, x, y, wallType, wallWidth, mapEditApplet.SHAPE_ADD_OR_UPDATE);
                        updateParametersDrag(w);                        
                        break;
                   
                    case Arc.OUTER_WALL_ARC:
                        Arc arc = null;
                        shapes.add(arc=new Arc(lastX, lastY, x, y, Arc.INITIAL_ARC_RADIUS, wallType, wallWidth));
                        editArc=arc;                        
                        updateParametersDrag(arc);
                        break;                      
                }//switch wallType                               
                repaint();
            }
        }//drawing mode
    }
    
    
    public void mouseMoved(MouseEvent e){
        updateCoordinates(e.getX(), e.getY());
        if(editArc!=null){
            int pX=e.getX(), pY=e.getY();
            int APx=pX-editArc.x, APy=pY-editArc.y;
            int qx=editArc.x2-editArc.x, qy=editArc.y2-editArc.y;
                       
            //editArc.radius=(int)(new Point2D.Float(editArc.x, editArc.y).distance(e.getX(), e.getY()) );            
            editArc.radius=(int)(2*(APx*qy-APy*qx)/(Math.sqrt(qx*qx+qy*qy)));
            //System.out.println("R: "+editArc.radius+" |APxq|: "+(APx*qy-APy*qx));
            repaint();
        }
    }//mouseMoved

    public void mouseClicked(MouseEvent e){
        //if left mouse button is clicked draw window or door if checkbox is selected
        if( (e.getModifiers() & InputEvent.BUTTON1_MASK)== InputEvent.BUTTON1_MASK )
        {
            if( !edit ){
                if(wallType==Text.TEXT){
                    String text=mapEditApplet.textFieldText.getText();
                    text.trim();
                    if(!text.equals("")) shapes.add(new Text(e.getX(), e.getY(), text));
                    repaint();
                }//text
                
                editArc=null;
            }//!edit
            
            return;
        }
                
        //if right mouse button is clicked, remove selected Shape
        if( (e.getModifiers() & InputEvent.BUTTON3_MASK)== InputEvent.BUTTON3_MASK )
        {
            if(selectShape(e)){//click should be in some Shape to delete
                if(selectedShape instanceof Wall){                    
                    //tohle bere v uvahu Grid
                    //((Wall)selectedShape).addHole(lastX, lastY, wallType, wallWidth);                                        
                    mapEditApplet.updateWallOnDilewa(((Wall)selectedShape).ID, ((Wall)selectedShape).x, ((Wall)selectedShape).y, ((Wall)selectedShape).x2, ((Wall)selectedShape).y2, ((Wall)selectedShape).type, ((Wall)selectedShape).thickness, mapEditApplet.SHAPE_DELETE);
                    delete();
                    repaint();
                }//Wall
                else {
                    delete();
                    repaint();
                }//else
            }

        }//right mouse button
    }
    
    
    public void mouseExited(MouseEvent e){}
    public void mouseEntered(MouseEvent e){}     

    
    //updates selected Shape
    public void updateSelected(){  
        if(selectedShape!=null)
            if(selectedShape instanceof Wall){
                Wall w=(Wall)selectedShape;
                w.type=wallType;
                w.height=Transform.mm2point(wallWidth);
                repaint();
            }
    }
    
    
    //updates parameters in mapEditApplet to reflect selected Shape
    public void updateParameters(Shape s, int x, int y){  
        if(s==null) return;
        if(s instanceof Wall){                                   
            Wall w=(Wall)s;            
            mapEditApplet.updateParametersWall(w);            
            updateParametersDrag(w);            
        }//Wall        
        if(s instanceof Text) mapEditApplet.radioButtonText.doClick();
        
    }//updateParameters
    
    
    //updates parameters in mapEditApplet to reflect selected Shape while 
    //dragging (no type and with change)
    public void updateParametersDrag(Shape s){  
        if(s==null) return;
        if(s instanceof Wall){
            Wall w=(Wall)s;
            mapEditApplet.textFieldWallLength.setText((new Integer(Transform.point2mm(w.width))).toString());
            mapEditApplet.textFieldX1.setText((new Integer(Transform.point2mm(w.x))).toString());
            mapEditApplet.textFieldY1.setText((new Integer(Transform.point2mm(w.y))).toString());
            mapEditApplet.textFieldX2.setText((new Integer(Transform.point2mm(w.x2))).toString());
            mapEditApplet.textFieldY2.setText((new Integer(Transform.point2mm(w.y2))).toString());
        }//Wall        
    }//updateParametersDrag
    
    
    //updates X, Y actual coordinates of the mouse
    public void updateCoordinates(int x, int y){
        Integer X=new Integer(Transform.point2mm(x));
        Integer Y=new Integer(Transform.point2mm(y));
        mapEditApplet.textFieldX.setText(X.toString());
        mapEditApplet.textFieldY.setText(Y.toString());        
    }//updateCoordinates
    
        
    public int gridCoord(int x){
        return ((int)Math.round((float)x/10.0)*10); 
    }
    
    // Updates the coordinates representing the location of the current rectangle.
    public void updateLocation(MouseEvent e){                
        if(selectedShape!=null)
        if(selectedShape instanceof Rectangle) {
            if(mapEditApplet.checkBoxGrid.isSelected()) 
                ((Rectangle)selectedShape).setLocation(gridCoord(lastX + e.getX()), gridCoord(lastY + e.getY()));
            else 
                ((Rectangle)selectedShape).setLocation(lastX + e.getX(), lastY + e.getY());
            //Dilewa update
            if(selectedShape instanceof Wall){
                mapEditApplet.updateWallOnDilewa(((Wall)selectedShape).ID, ((Wall)selectedShape).x, ((Wall)selectedShape).y, ((Wall)selectedShape).x2, ((Wall)selectedShape).y2, ((Wall)selectedShape).type, ((Wall)selectedShape).thickness, mapEditApplet.SHAPE_ADD_OR_UPDATE);
            }
        }
        /*
         * Updates the label to reflect the location of the
         * current rectangle
         * if checkRect returns true; otherwise, returns error message.
         */
/*        if (checkRect()) {
            //SwingShapeMover.label.setText("Rectangle located at " +
            //                                 rect.getX() + ", " +
            //                                 rect.getY());
        } else {
            //SwingShapeMover.label.setText("Please don't try to "+
            //                                 " drag outside the area.");
        }
*/
        repaint();
    }//updateLocation

    
    /**
        Draw the grid
    */
    public void drawGrid(Graphics g, Color color, int w, int h)
    {
        g.setColor(color);
        for(int x=0; x<=w; x+=10)
            g.drawLine(x,0,x,h);
        for(int y=0; y<=h; y+=10)
            g.drawLine(0,y,w,y);
    }


    
    public void paintComponent(Graphics g){
        super.paintComponent(g);        
        
        Graphics2D g2 = (Graphics2D)g;                
        Shape s=null;
        Iterator iter=null;

        Dimension dim = getSize();
        int w = (int)dim.getWidth();
        int h = (int)dim.getHeight();
        
        // Clears the drawing area
        g2.setPaint(Color.white);
        g2.fillRect(0, 0, w, h);

        //should we draw grid?
        if(mapEditApplet.checkBoxGrid.isSelected()){
            drawGrid(g, Color.lightGray, w, h);
        }
                                                  
        // Draws and fills all the shapes
        iter=shapes.iterator();        
        g2.setStroke(new BasicStroke(2.0f));
//        g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
        while(iter.hasNext()){         
            s=(Shape)iter.next();
            if(s instanceof Wall){ 
                if(s==selectedShape) ((Wall)s).drawFillSelected(g2);
                else {
                    ((Wall)s).drawFill(g2);
                }
            }//wall
            
            if(s instanceof Text){ 
                ((Text)s).draw(g2);
            }//text
            
        }//while                    
              
    }//paintComponent

    
    public Shape findShape(int id){
        Shape s=null;
        Iterator iter=null;
        iter=shapes.iterator();
        while(iter.hasNext()){         
            s=(Shape)iter.next();
            if(s instanceof Wall){ 
                if( ((Wall)s).ID==id ) return(s);
            }//wall
            
//            if(s instanceof Text){ 
//                ((Text)s).draw(g2);
//            }//text
            
        }//while                    
        return(null);      
    };//findShape

    
    
    /*
     Checks if the rectangle is contained within the applet window.  If the rectangle
     is not contained withing the applet window, it is redrawn so that it is adjacent
     to the edge of the window and just inside the window.
    */
/*    boolean checkRect(){

	       if (area == null) {
			return false;
	       }

               if(area.contains(rect.x, rect.y, 100, 50)){
                        return true;
                }
                int new_x = rect.x;
                int new_y = rect.y;
        
                if((rect.x+100)>area.getWidth()){
                        new_x = (int)area.getWidth()-99;
                }
                if(rect.x < 0){
                        new_x = -1;  
                }
                if((rect.y+50)>area.getHeight()){
                        new_y = (int)area.getHeight()-49;
                }
                if(rect.y < 0){
                        new_y = -1;
                }
                rect.setLocation(new_x, new_y);
                return false;
    }//checkRect
*/
    
}//class EditPanel
