/*
 * Wall.java
 *
 * Created on 12. 3. 2003
 */

/**
 *
 * @author  Martin Cadik - cadikm@cs.felk.cvut.cz
 * @version 1.0
 */

import java.util.*;
import java.awt.*;
import java.awt.geom.*;

public class Wall extends Rectangle {
    public static final int WALL_ROAD=1;
    public static final int WALL_RIVER=2;
    public static final int WALL_BRIDGE=30;
    
    public int ID=0; //unique ID for Dilewa
    public int type=WALL_ROAD;
    public double rot=0.0; //rotation
    /** x coordinate of the ending point */
    public int x2=0;
    /** y coordinate of the ending point */
    public int y2=0;
    /** Skutecna sirka prvku nezavisla na meritku */
    public int thickness=270;
    
    /** Creates new Wall */
    public Wall() {
        super();
    }
    
    
    public Wall(int x, int y, int width, int height){
        super(x, y, width, height);
    }

    
    /** 
        Creates new Wall 
        thickness is in milimeters, left upper point is lexicographically sorted
        against right lower point 
    */
    public Wall(int ID, int x1, int y1, int x2, int y2, int type, int thickness){
        this(x1, y1, (int)(new Point2D.Float(x1, y1).distance(x2,y2)), Transform.mm2point(thickness));
        this.thickness=thickness;
        this.ID=ID;
        if(!( (x1<x2) || ((x1==x2)&&(y1<=y2)) )) {
            this.x=x2; this.y=y2;
            x2=x1; y2=y1; x1=this.x; y1=this.y; 
        
        }                
        this.type=type;
        rot=Math.atan((double)(y2-y1)/(double)(x2-x1));
        if(x2<x1) rot+=Math.PI;
        this.x2=x2;
        this.y2=y2;
    }

      
    public Object clone(){
        Wall w=new Wall(++ID, x, y, x2, y2, type, thickness);
        return(w);
    }
    
    public boolean contains(double x, double y){
        AffineTransform at=new AffineTransform();
        at.setToTranslation(this.x,this.y);
        at.rotate(rot);
        at.translate(-this.x,-this.y);
        
        //y should be in the middle (axis) of the rectangle (not on top line)
        //at.translate(0,-this.height/2);
        
        Shape s=at.createTransformedShape(this);        
        //System.out.println("Transformed shape: "+s);
        return(s.contains(x, y));
    }
    
    public void setLocation(int x, int y){
        x2+=(x-this.x);
        y2+=(y-this.y);
        super.setLocation(x,y);        
    }//setLocation
    
    public void draw(Graphics2D g2) {
        AffineTransform at=new AffineTransform();
        at.setToTranslation(x,y);
        at.rotate(rot);
        at.translate(-x,-y);
        
        //y should be in the middle (axis) of the rectangle (not on top line)
        //at.translate(0,-this.height/2);
        
        if(this.type==WALL_ROAD) g2.setPaint(Color.gray);
        if(this.type==WALL_RIVER) g2.setPaint(Color.blue);
        if(this.type==WALL_BRIDGE) g2.setPaint(Color.black);
        g2.draw(at.createTransformedShape(this));
        g2.setPaint(Color.lightGray);       
    }//draw

    
    public void fill(Graphics2D g2) {
        AffineTransform at=new AffineTransform();
        at.setToTranslation(x,y);
        at.rotate(rot);
        at.translate(-x,-y);

        //y should be in the middle (axis) of the rectangle (not on top line)
        //at.translate(0,-this.height/2);
        
        if(this.type==WALL_ROAD) g2.setPaint(Color.red);
        if(this.type==WALL_RIVER) g2.setPaint(Color.cyan);
        if(this.type==WALL_BRIDGE) g2.setPaint(Color.gray);

        g2.fill(at.createTransformedShape(this));
    }//fill


    public void drawFill(Graphics2D g2) {
        AffineTransform at=new AffineTransform();
        at.setToTranslation(x,y);
        at.rotate(rot);
        at.translate(-x,-y);        
        //y should be in the middle (axis) of the rectangle (not on top line)
        //at.translate(0,-this.height/2);        
        if(this.type==WALL_ROAD) g2.setPaint(Color.gray);
        if(this.type==WALL_RIVER) g2.setPaint(Color.blue);
        if(this.type==WALL_BRIDGE) g2.setPaint(Color.black);
        g2.draw(at.createTransformedShape(this));
        if(this.type==WALL_ROAD) g2.setPaint(Color.red);
        if(this.type==WALL_RIVER) g2.setPaint(Color.cyan);
        if(this.type==WALL_BRIDGE) g2.setPaint(Color.gray);
        g2.fill(at.createTransformedShape(this));               
    }//drawFill

    
    /* zvyrazneni prave vybrane zdi */
    public void drawFillSelected(Graphics2D g2) {
        AffineTransform at=new AffineTransform();
        at.setToTranslation(x,y);
        at.rotate(rot);
        at.translate(-x,-y);        
        //y should be in the middle (axis) of the rectangle (not on top line)
        //at.translate(0,-this.height/2);        
        g2.setPaint(Color.gray);
        g2.draw(at.createTransformedShape(this));
        g2.setPaint(Color.green);
        g2.fill(at.createTransformedShape(this));
    }//drawFillSelected

}//class wall